local cls = CombatLogSearchButton

local MAX_HITS = 100 -- maximum number of search hits to display before stopping search
local MAX_LINES = 100 -- maximum number of lines to search in one step

-- Blizzard_CombatLog is Load on Demand but can be loaded before PLAYER_LOGIN
function cls:PLAYER_LOGIN()
	local parent = CombatLogQuickButtonFrame_Custom
	if not parent then
		self:RegisterEvent("ADDON_LOADED") -- it's not loaded yet, watch for it to load
	else
		self:UnregisterAllEvents() -- unnecessary, but just to say no events registered >:D
		self:SetParent(parent) -- place search button on the log and align editbox to right edge of log
		self:SetPoint("TOPRIGHT",parent,"TOPRIGHT")
		local filterButton = CombatLogQuickButtonFrame_CustomAdditionalFilterButton
		filterButton:ClearAllPoints()
		filterButton:SetPoint("RIGHT",self,"LEFT",8,0)
		self.EditBox:SetPoint("BOTTOMRIGHT",parent,"TOPRIGHT",3,1)
	end
end

-- in the event Blizzard_CombatLog was not loaded at PLAYER_LOGIN, watch for it to load
function cls:ADDON_LOADED(addon)
	if CombatLogQuickButtonFrame_CustomAdditionalFilterButton then
		self:PLAYER_LOGIN() -- go through the login motions again, this time we know Blizzard_CombatLog is loaded
	end
end

-- OnClick of the button shows or hides the editbox
function cls:ToggleEditBox()
	if self.EditBox:IsVisible() then
		self.EditBox:Hide()
	else
		self.EditBox:SetText("")
		self.EditBox:Show()
		self.EditBox:SetFocus(true)
	end
end

-- helper functions to escape magic characters and make search case insensitive
local function literal(c) return "%"..c end
local function caseinsensitive(c) return format("[%s%s]",c:lower(),c:upper()) end

-- does an AddMessage to the log
function cls:AddMessage(...)
	cls.pauseCombatLog = nil -- unpause to let our stuff through
	COMBATLOG:AddMessage(...)
	cls.pauseCombatLog = true -- re-pause to stop log so results can be read
end

-- when Enter hit in the editbox we grab its text and start looking
function cls:Search()
	cls.pattern = (self:GetText() or ""):trim()
	if cls.pattern and cls.pattern:len()>0 then
	
		cls.pattern = cls.pattern:gsub("[%(%)%.%%%+%-%*%?%[%]%^%$]",literal):gsub("%a",caseinsensitive)
		cls:SetupFilter() -- create/update filter (for formatting)
		CombatLogSetCurrentEntry(0,true) -- 0 is the most recent entry (last one listed)

		-- set up progress bar
		CombatLogQuickButtonFrame_CustomProgressBar:SetMinMaxValues(0,max(1,CombatLogGetNumEntries(true)))
		CombatLogQuickButtonFrame_CustomProgressBar:SetValue(0)
		CombatLogQuickButtonFrame_CustomProgressBar:Show()

		-- stop any existing refiltering and set up log to be paused
		cls:CheckHookSetup() -- enable pausing of log if we haven't yet
		if CombatLogUpdateFrame.refiltering then -- abort any default refiltering happening
			CombatLogUpdateFrame:SetScript("OnUpdate",nil)
		end

		cls.entry = 0 -- increments for each entry tested (for progress bar value)
		cls.count = 0 -- increments for each search hit encountered
		CombatLogUpdateFrame.refiltering=true
		COMBATLOG:Clear() -- waiting until a hit found to clear log, in case something mispelled they can scroll up

		cls:AddMessage("Searching...")
		cls:SetScript("OnUpdate",cls.SearchStep) -- do another SearchStep next frame
	else -- user hit enter with nothing in box, just hide it
		cls.EditBox:Hide()
	end
end

-- strips control codes out of a log entry for comparing with search text
local function stripcodes(text)
	return text:gsub("\124c%x%x%x%x%x%x%x%x",""):gsub("\124r",""):gsub("\124H.-\124h(.-)\124h","%1"):gsub("{.-}","")
end

-- this OnUpdate mimics the default's over-time deploy of combat log lines, one search hit per
-- frame with a progress bar to indicate how far along it is.
function cls:SearchStep()
	local firstEntry = cls.entry
	local text,r,g,b
	while CombatLogGetCurrentEntry(true) do -- looking for next search hit
		cls.entry = cls.entry + 1
		text,r,g,b = CombatLog_OnEvent(cls.Filter,CombatLogGetCurrentEntry(true))
		if text then
			local plaintext = stripcodes(text)
			if plaintext:match(cls.pattern) then -- we found a hit
				if cls.count==0 then
					COMBATLOG:Clear() -- if this is first hit, clear the "Searching..." from the log
				end
				cls:AddMessage(text,r,g,b,nil,true)
				cls.count = cls.count + 1
				CombatLogAdvanceEntry(-1,true)
				CombatLogQuickButtonFrame_CustomProgressBar:SetValue(cls.entry)
				if cls.count<MAX_HITS then -- only going to search for 100 hits
					return true -- something found, come back next frame for another search hit
				else
					break -- leave while loop if we've exceeded MAX_HITS
				end
			end
		end
		CombatLogAdvanceEntry(-1,true) -- move to next log entry
		if cls.entry-MAX_LINES>firstEntry then -- if we've searched enough for one frame
			CombatLogQuickButtonFrame_CustomProgressBar:SetValue(cls.entry)
			return -- we've searched enough, come back for next frame for another group of lines
		end
	end
	-- if we reached here, we're done searching; shut down the OnUpdate and do cleanup
	cls:SetScript("OnUpdate",nil)
	CombatLogQuickButtonFrame_CustomProgressBar:Hide()
	CombatLogUpdateFrame.refiltering = nil
	if not text then
		cls:AddMessage(format("%s%s matches found in %d log entries.",cls.count or "No",(cls.count and cls.count>=MAX_HITS) and "+" or "",cls.entry))	
	else
		cls:AddMessage(format("%s%s matches found in %d log entries back to %s.",cls.count or "No",(cls.count and cls.count>=MAX_HITS) and "+" or "",cls.entry,text:match("^(.-)>")))	
	end
	return false -- only relevant the first time it's called in cls:Search(), means nothing was found
end

-- when editbox loses focus, unpause queue and refilter
-- if button not on screen (went to another chat tab) just hide editbox
function cls:OnEditFocusLost()
	if cls.pauseCombatLog and cls:IsVisible() then
		Blizzard_CombatLog_Refilter() -- unpause built in here
	else
		CombatLogQuickButtonFrame_CustomProgressBar:Hide()
	end
	cls.pauseCombatLog = nil
	cls:SetScript("OnUpdate",nil)
	self:Hide()
end

-- when first hit of first search happens, this will set up hooks necessary to pause the log
function cls:CheckHookSetup()
	if not cls.hooksHappened then
		-- hook refilter to automatically unpause the log
		local oldRefilter =  Blizzard_CombatLog_Refilter
		function Blizzard_CombatLog_Refilter(...)
			cls.pauseCombatLog = nil -- unpause log
			cls:SetScript("OnUpdate",nil) -- and stop any searches happening
			return oldRefilter(...)
		end
		-- hook COMBATLOG:AddMessage to not add a message if log paused
		local oldAddMessage = COMBATLOG.AddMessage
		function COMBATLOG.AddMessage(...)
			if not cls.pauseCombatLog then
				return oldAddMessage(...)
			end
		end
		cls.hooksHappened = true
	end
end

-- combatlog functions are using ignoreFilter flag, but we still need a filter to format (timestamp etc)
-- so one is created here with verbose and unitColoring updated to currently active filter
function cls:SetupFilter()
	if not cls.Filter then
		-- start with a copy of default filter settings
		cls.Filter = { quickButtonDisplay = {},
			settings = CopyTable(COMBATLOG_DEFAULT_SETTINGS),
			colors = CopyTable(COMBATLOG_DEFAULT_COLORS),
			filters = { {eventList=Blizzard_CombatLog_GenerateFullEventList(), sourceFlags=Blizzard_CombatLog_GenerateFullFlagList(true)},
									{eventList=Blizzard_CombatLog_GenerateFullEventList(), destFlags=Blizzard_CombatLog_GenerateFullFlagList(true)}}
			}
		-- with some tweaks (timestamp and show auras) that aren't in default
		cls.Filter.settings.timestamp = true
		cls.Filter.settings.hideBuffs = false
		cls.Filter.settings.hideDebuffs = false
	end
	-- make verbose and unit coloring settings mimic what user has for their active filter
	cls.Filter.settings.fullText = Blizzard_CombatLog_CurrentSettings.settings.fullText
	cls.Filter.settings.unitColoring = Blizzard_CombatLog_CurrentSettings.settings.unitColoring
end
